/******************************************************************************

File: encap.h

Author: Samuel Jero

Date: 12/2010

Description:  Declarations of encapsulation functions for ltp2tcp

******************************************************************************/

#ifndef ENCAP_H_
#define ENCAP_H_
#include <netinet/if_ether.h>
#include <netinet/ip.h>

/*


*/

/* Encapsulation Operations*/
struct encap_ops{
	/* Handle all headers up to LTP:
	 * 	h is the libpcap header. It should be returned with the Link+IP length in it.
	 *  odata is the packet being processed starting at the Link header. It should be returned starting at the LTP header.
	 *  ndata is the new packet. It should be returned with a Link+IP header in it.
	 *  olength is the length of the packet being processed. It should be returned as the length of the LTP segment.
	 *  nlength is the remaining length of the new packet. The Link+IP headers should be subtracted from it.
	 *
	 *  Other things to do:
	 *  1)IP must say that TCP is next header
	 *  2)Initialize encapsulation private data as needed (typically store h and Link+IP header for fin() method)
	 */
	int (*pre)(struct pcap_pkthdr *h, const u_char **odata, u_char **ndata, int* olength, int* nlength);
	/*
	 * Do any needed post processing on packet:
	 *  tlen is the length of the TCP segment (data+header)
	 *  data is the new packet starting at the link header
	 *
	 *  Typically update IP length
	 */
	int (*post)(int tlen, u_char *data);
	/*
	 * Add a TCP initial Threeway handshake to the connection:
	 *  h is the libpcap header of the current packet.
	 *
	 *  This means create three packets:
	 *  	SYN in current Direction (SEQ=0,ACK=0)
	 *  	SYN,ACK in opposite Direction (SEQ=0,ACK=1)
	 *  	ACK in current Direction (SEQ=1,ACK=1)
	 *
	 *  You typically need Link+IP header from private data
	 *  Output packets to state.out
	 */
	int (*handshake)(struct pcap_pkthdr *h);
	/*
	 * Add TCP closing handshake:
	 * 	This means create three packets:
	 * 		FIN,ACK in current Direction (SEQ=state.seq_num+1, ACK=state.ack_num)
	 * 		FIN,ACK in opposite Direction (SEQ=state.ack_num, ACK=state.seq_num+2)
	 * 		ACK		in current Direction (SEQ=state.seq_num+2,ACK=state.ack_num+1)
	 *
	 * 	You typically need the libpcap header and the Link+IP header from private data
	 *  Output packets to state.out
	 */
	int (*fin)();
};


/*Encapsulation Operations Structures*/
extern struct encap_ops udp_encap;
extern struct encap_ops dccp_encap;
extern struct encap_ops sll_encap;


/*Encapsulation Selector*/
void encap_sel(char* string);

/*Standard Ethernet, IP encapsulation structures and functions*/
struct eip4_en_p{
	struct pcap_pkthdr	header;
	u_char				od[sizeof(struct ether_header)+sizeof(struct iphdr)];
	int					first;
};
int fill_eip4_encap(struct eip4_en_p *eip, const u_char* data, int dlen, struct pcap_pkthdr *h);
int eip4_post(struct eip4_en_p *eip, int tlen, u_char* data);
int eip4_handshake(struct eip4_en_p *eip, struct pcap_pkthdr *h);
int eip4_fin(struct eip4_en_p *eip);

#endif /* ENCAP_H_ */
